{ config, pkgs, lib, ... }:

{
  imports =
    [
      ./hardware-configuration.nix
      ../../configuration/common
      ../../configuration/router
    ];

  boot.loader.grub.enable = true;
  boot.loader.grub.version = 2;
  boot.loader.grub.device = "/dev/sda";

  networking.hostName = "porter";

  networking.useDHCP = false;
  # Local Router IPs
  networking.interfaces.lo.ipv6.addresses = [
    { address = "fd00:152:152:102::1"; prefixLength = 64; }
    { address = "fd00:152:152::1"; prefixLength = 128; } # Anycast
  ];
  networking.interfaces.lo.ipv4.addresses = [
    { address = "10.152.102.1"; prefixLength = 24; }
    { address = "10.152.0.1"; prefixLength = 32; } # Anycast
  ];
  # Network
  networking.interfaces.ens3.ipv4.addresses = [ { address = "188.34.158.206"; prefixLength = 32; } ];
  networking.interfaces.ens3.ipv6.addresses = [ { address = "2a01:4f8:c010:4c92::1"; prefixLength = 64; } ];
  networking.defaultGateway = { address = "172.31.1.1"; interface = "ens3"; };
  networking.defaultGateway6 = { address = "fe80::1"; interface = "ens3"; };
  networking.nameservers = [ "213.133.98.98" "213.133.99.99" "213.133.100.100" ];

  networking.wireguard.enable = true;
  networking.wireguard.interfaces = {
    wg-carbon = {
      ips = [ "fe80::138:1/64" "169.254.138.1/24" ];
      peers = [ {
          allowedIPs = [ "0.0.0.0/0" "::/0" ];
          publicKey = "ezemzjfG6OZ3rHcCvuh+N1VfaIgbijvFbvvRmhasu2E=";
      } ];
      listenPort = 50138;
      allowedIPsAsRoutes = false;
      privateKeyFile = "/var/src/secrets/wireguard/wg-carbon";
    };
    wg-nonat = {
      ips = [ "fe80::1337:1/64" "169.254.137.1/24" ];
      peers = [ {
          allowedIPs = [ "0.0.0.0/0" "::/0" ];
          endpoint = "nonat.net.clerie.de:51337";
          publicKey = "Z5HltUKBSOzePqZCJjSsJPZ3UxGjFR4a5Vxmm+ePNRk=";
      } ];
      listenPort = 51337;
      allowedIPsAsRoutes = false;
      privateKeyFile = "/var/src/secrets/wireguard/wg-nonat";
    };
  };

  networking.firewall.allowedUDPPorts = [ 50138 51337 ];

  services.bird2.enable = true;
  services.bird2.config = ''
  router id ${ (lib.head config.networking.interfaces.lo.ipv4.addresses).address };

  ipv6 table ospf6;
  ipv4 table ospf4;
  ipv6 table bgp6;
  ipv4 table bgp4;

  protocol direct {
    interface "lo";
    ipv6 {
      table ospf6;
    };
    ipv4 {
      table ospf4;
    };
  }

  protocol kernel kernel_6 {
    ipv6 {
      table ospf6;
      export filter {
        krt_prefsrc=fd00:152:152:102::1;
        accept;
      };
      import none;
    };
  }

  protocol kernel kernel_4 {
    ipv4 {
      table ospf4;
      export filter {
        krt_prefsrc=10.152.102.1;
        accept;
      };
      import none;
    };
  }

  protocol ospf v3 ospf_6 {
    ipv6 {
      table ospf6;
      import all;
      export all;
    };
    area 0 {
      interface "wg-carbon" {
        cost 80;
        type pointopoint;
      };
      interface "wg-nonat" {
        cost 80;
        type pointopoint;
      };
    };
  }

  protocol ospf v3 ospf_4 {
    ipv4 {
      table ospf4;
      import all;
      export all;
    };
    area 0 {
      interface "wg-carbon" {
        cost 80;
        type pointopoint;
      };
      interface "wg-nonat" {
        cost 80;
        type pointopoint;
      };
    };
  }

  template bgp ibgp6 {
    local as 4200002574;
    graceful restart on;
    source address ${ (lib.head config.networking.interfaces.lo.ipv6.addresses).address };
    ipv6 {
      table bgp6;
      next hop self;
      import keep filtered;
      import all;
      export all;
    };
  }

  template bgp ibgp4 {
    local as 4200002574;
    graceful restart on;
    source address ${ (lib.head config.networking.interfaces.lo.ipv4.addresses).address };
    ipv4 {
      table bgp4;
      next hop self;
      import keep filtered;
      import all;
      export all;
    };
  }

  protocol bgp bgp_carbon from ibgp6 {
    neighbor fd00:152:152:104::1 as 4200002574;
  }

  protocol bgp bgp_nonat from ibgp6 {
    neighbor fd00:152:152:103::1 as 4200002574;
  }

  protocol device {
    scan time 10;
  }
  '';

  system.stateVersion = "21.03";
}
